﻿Imports System.Drawing.Drawing2D
Imports System.IO

Public Class FormMain

#Region "Private Propertys"

    Private _extensions As New List(Of String)
    Private ReadOnly Property Extensions As List(Of String)
        Get
            Return _extensions
        End Get
    End Property

    Private _allDirectories As New Boolean
    Private Property AllDirectories As Boolean
        Get
            Return _allDirectories
        End Get
        Set(value As Boolean)
            _allDirectories = value
        End Set
    End Property

#End Region

#Region "Private Events"

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Text = Application.ProductName
        ListViewDBImageView.TileSize = New Size(260, 260)
        _extensions.AddRange({".png", ".jpg", ".gif"})
        SelectedToolStripMenuItem.Enabled = False
        AllToolStripMenuItem.Enabled = False
        RemoveToolStripMenuItem.Enabled = False
        SetStatusLabel()
    End Sub

    Private Sub SelectDirectoryToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles SelectDirectoryToolStripMenuItem.Click
        Using folderBrowserDialog As New FolderBrowserDialog()
            With folderBrowserDialog
                .Description = "Select Folder"
                .ShowNewFolderButton = False
                .RootFolder = Environment.SpecialFolder.MyComputer
                If folderBrowserDialog.ShowDialog() = DialogResult.OK Then
                    Try
                        For Each file As String In Directory.EnumerateFiles(.SelectedPath, "*", If(AllDirectories, SearchOption.AllDirectories, SearchOption.TopDirectoryOnly))
                            Dim fi As New FileInfo(file)
                            If Extensions.Contains(fi.Extension.ToLower) Then
                                Try
                                    Dim image As Image = Image.FromFile(fi.FullName)
                                    Dim thumbnailSize As Size = GetThumbnailSize(image)
                                    Dim item As New ImageListViewItem
                                    With item
                                        .Tag = fi.Name
                                        .ToolTipText = fi.FullName
                                        .Bitmap = CType(image.GetThumbnailImage(thumbnailSize.Width, thumbnailSize.Height, Nothing, IntPtr.Zero), Bitmap)
                                    End With
                                    image.Dispose()
                                    ListViewDBImageView.Items.Add(item)
                                Catch oomex As OutOfMemoryException
                                End Try
                            End If
                        Next
                    Catch uaex As UnauthorizedAccessException
                    End Try
                End If
            End With
        End Using
        SelectedToolStripMenuItem.Enabled = False
        AllToolStripMenuItem.Enabled = True
        RemoveToolStripMenuItem.Enabled = True
    End Sub

    Private Sub AddToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles AddToolStripMenuItem.Click
        Using openFileDialog As New OpenFileDialog
            With openFileDialog
                .Filter = "Images (*.png;*.jpg;*.gif)|*.png;*.jpg;*.gif"
                .Multiselect = True
                If openFileDialog.ShowDialog = System.Windows.Forms.DialogResult.OK Then
                    For Each fileName As String In openFileDialog.FileNames
                        Dim fi As New FileInfo(fileName)
                        Dim image As Image = Image.FromFile(fi.FullName)
                        Dim thumbnailSize As Size = GetThumbnailSize(image)
                        Dim item As New ImageListViewItem()
                        With item
                            .Tag = fi.Name
                            .ToolTipText = fi.FullName
                            .Bitmap = CType(image.GetThumbnailImage(thumbnailSize.Width, thumbnailSize.Height, Nothing, IntPtr.Zero), Bitmap)
                        End With
                        image.Dispose()
                        ListViewDBImageView.Items.Add(item)
                    Next
                End If
            End With
        End Using
        SelectedToolStripMenuItem.Enabled = False
        AllToolStripMenuItem.Enabled = True
        RemoveToolStripMenuItem.Enabled = True
    End Sub

    Private Sub ListViewDBImageView_DrawItem(ByVal sender As Object, ByVal e As DrawListViewItemEventArgs) Handles ListViewDBImageView.DrawItem
        Dim item As ImageListViewItem = DirectCast(e.Item, ImageListViewItem)
        Dim imageWidth As Integer = Math.Min(256, item.Bitmap.Width)
        Dim imageHeight As Integer = Math.Min(256, item.Bitmap.Height)
        e.Graphics.CompositingQuality = CompositingQuality.HighQuality
        e.Graphics.PixelOffsetMode = PixelOffsetMode.HighQuality
        e.Graphics.SmoothingMode = SmoothingMode.HighQuality
        e.Graphics.Clip = New Region(e.Bounds)
        ' Background
        Dim backgroundBrush As New SolidBrush(SystemColors.Window)
        e.Graphics.FillRectangle(backgroundBrush,
                                 New Rectangle(e.Bounds.X,
                                               e.Bounds.Y,
                                               e.Bounds.Width,
                                               e.Bounds.Height))
        ' Image
        e.Graphics.DrawImage(item.Bitmap,
                             New Rectangle(e.Bounds.X + (e.Bounds.Width - imageWidth) \ 2,
                                           e.Bounds.Y + (e.Bounds.Height - imageHeight) \ 2,
                                           imageWidth,
                                           imageHeight),
                             New Rectangle(Point.Empty, item.Bitmap.Size),
                             GraphicsUnit.Pixel)
        ' Footer Background
        Dim footerBackgroundBrush As New SolidBrush(Color.FromArgb(200, 0, 0, 0))
        e.Graphics.FillRectangle(footerBackgroundBrush,
                                     New Rectangle(e.Bounds.X,
                                                   e.Bounds.Y + e.Bounds.Height - 20,
                                                   e.Bounds.Width,
                                                   20))
        ' Footer Text
        TextRenderer.DrawText(e.Graphics,
                              item.Tag.ToString,
                              Font,
                              New Rectangle(e.Bounds.Left,
                                            e.Bounds.Bottom - Font.Height - 4,
                                            e.Bounds.Width,
                                            Font.Height),
                              Color.White)
        ' Border
        Dim brush As New SolidBrush(SystemColors.ControlLight)
        If e.Item.Selected Then brush = New SolidBrush(SystemColors.MenuHighlight)
        Dim pen As New Pen(brush, 4)
        e.Graphics.DrawRectangle(pen, e.Bounds)
        '
        SetStatusLabel()
    End Sub

    Private Sub ListViewDBImageView_SelectedIndexChanged(sender As Object, e As EventArgs) Handles ListViewDBImageView.SelectedIndexChanged
        If ListViewDBImageView.SelectedItems.Count = 0 Then
            SelectedToolStripMenuItem.Enabled = False
        Else
            SelectedToolStripMenuItem.Enabled = True
        End If
    End Sub

    Private Sub AllToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles AllToolStripMenuItem.Click
        ClearListView(ListViewDBImageView)
        If ListViewDBImageView.Items.Count = 0 Then
            RemoveToolStripMenuItem.Enabled = False
        End If
        SetStatusLabel()
    End Sub

    Private Sub SelectedToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles SelectedToolStripMenuItem.Click
        For Each index As Integer In From i In ListViewDBImageView.SelectedIndices.Cast(Of Integer)() Order By i Descending
            CType(ListViewDBImageView.Items(index), ImageListViewItem).Bitmap.Dispose()
            ListViewDBImageView.Items.RemoveAt(index)
        Next
        If ListViewDBImageView.Items.Count = 0 Then
            RemoveToolStripMenuItem.Enabled = False
        End If
    End Sub

    Private Sub IncludeSubDirectorysToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles IncludeSubDirectorysToolStripMenuItem.Click
        AllDirectories = IncludeSubDirectorysToolStripMenuItem.Checked
    End Sub

#End Region

    Private Function GetThumbnailSize(ByVal original As Image) As Size
        Const maxDimensionSize As Integer = 256
        Dim originalWidth As Integer = original.Width
        Dim originalHeight As Integer = original.Height
        Dim factor As Double
        If originalWidth > originalHeight Then
            factor = maxDimensionSize / originalWidth
        Else
            factor = maxDimensionSize / originalHeight
        End If
        Return New Size(CInt(Math.Truncate(originalWidth * factor)), CInt(Math.Truncate(originalHeight * factor)))
    End Function

    Private Sub SetStatusLabel()
        Dim ItemsCount As Integer = ListViewDBImageView.Items.Count
        Dim ItemsCountText As String = "Images"
        If ItemsCount = 1 Then ItemsCountText = ItemsCountText.Remove(ItemsCountText.Length - 1)
        ToolStripStatusLabelStatus.Text = String.Format("{0} {1}", ItemsCount, ItemsCountText)
    End Sub

    Private Sub ClearListView(ByVal listView As ListView)
        For Each item As ListViewItem In listView.Items
            CType(item, ImageListViewItem).Bitmap.Dispose()
        Next
        listView.Items.Clear()
    End Sub

    Private Class ImageListViewItem
        Inherits ListViewItem

        Public Property Bitmap() As Bitmap

    End Class

End Class
